import logging

import requests
from msal import ConfidentialClientApplication

from odoo import _, fields, models
from odoo.exceptions import UserError

_logger = logging.getLogger(__name__)
ms_graph_site_url = "https://graph.microsoft.com/v1.0/sites"

root_site = "doc-erp"
tenant_id = "76d9117c-fdf5-4dff-9dd2-11a06fa7cd7e"
default_timeout = 1


class SharepointMixin(models.AbstractModel):
    _name = "sharepoint.mixin"
    _description = "Mixing to create scop drive in sharepoint"

    sharepoint_folder_id = fields.Char(
        "Identifiant di dossier dans sharepoint", default=None
    )
    sharepoint_folder_url = fields.Char(
        "Url sharepoint", default="https://cgscop.sharepoint.com/sites/doc-erp/SIOF"
    )

    def create_folder(self, site_id, drive_id, folder_name, parent_path=None):
        # Vérifie si on crée un dossier à la racine ou dans un sous-dossier
        if parent_path:
            url = (
                f"{ms_graph_site_url}/{site_id}/drives/{drive_id}"
                f"/root:/{parent_path}/{folder_name}"
            )
        else:
            url = f"{ms_graph_site_url}/{site_id}/drives/{drive_id}/root/children"

        data = {
            "name": folder_name,
            "folder": {},
            "@microsoft.graph.conflictBehavior": "rename",
        }

        response = requests.post(
            url,
            headers=self.make_request_headers(self.env.user),
            json=data,
            timeout=default_timeout,
        )

        if response.status_code == 201:
            self.sharepoint_folder_id = response.json().get("id")
            self.sharepoint_folder_url = response.json().get("webUrl")
        else:
            _logger.error(f"Erreur ({response.status_code}):", response.text)
            return None

    def init_sharepoint_directory(self):
        directory_name = self.name if self.name else "default_dir_name"
        root_site_data = self._get_root_site_data()
        site_id = root_site_data.get("id").split(",")[1]
        siof_drive = self._get_siof_drive(site_id=site_id)
        self.create_folder(
            site_id=site_id, drive_id=siof_drive.get("id"), folder_name=directory_name
        )

    def get_ms_graph_admin_token(self):
        """
        Authentification à MS Graph
        En sortie, la chaine requestHeaders constituée pour les futurs appels à l'API
        """

        authority = "https://login.microsoftonline.com/" + tenant_id
        scope = ["https://graph.microsoft.com/.default"]

        root_admin_user = self.env["ir.config_parameter"].get_param(
            "siof_sharepoint.admin_sharepoint_user_id", default=""
        )

        root_admin_pass = self.env["ir.config_parameter"].get_param(
            "siof_sharepoint.admin_sharepoint_secret_id", default=""
        )

        if not root_admin_user or not root_admin_pass:
            raise UserError(_("Sharepoint admin credentials not configured"))

        app = ConfidentialClientApplication(
            root_admin_user, authority=authority, client_credential=root_admin_pass
        )
        accesstoken = app.acquire_token_silent(scope, account=None)
        if not accesstoken:
            accesstoken = app.acquire_token_for_client(scopes=scope)
            if accesstoken.get("access_token"):
                _logger.info("New access token retreived....")
                return accesstoken["access_token"]
            else:
                _logger.error(
                    "Error acquiring authorization token. "
                    "Check your tenantID, clientID and clientSecret."
                )
                raise UserError(
                    f"{accesstoken.get("error")}\n{accesstoken.get("error_description")}"
                )
        else:
            _logger.info("Token retreived from MSAL Cache....")
        return

    def make_request_headers(self, connected_user):
        provider_id = connected_user.sudo().oauth_provider_id

        if provider_id.env.user.oauth_access_token:
            token = provider_id.env.user.oauth_access_token
        else:
            # test si superadmin
            token = self.get_ms_graph_admin_token()

        return {
            "Authorization": "Bearer " + token,
            "Content-type": "application/json",
        }

    def _get_root_site_data(self):
        """
        Retourne l'ID du Site
        """
        try:
            url = f"{ms_graph_site_url}/root:/sites/doc-erp"
            results = requests.get(
                url,
                headers=self.make_request_headers(self.env.user),
                timeout=default_timeout,
            ).json()
            error = results.get("error")
            if error:
                raise UserError(f"{error.get('code')}\n{error.get('message')}")
            return results
        except Exception:
            raise

    def _get_siof_drive(self, site_id):
        """
        Retourne l'ID du Site
        """

        try:
            url = f"{ms_graph_site_url}/{site_id}/drives"
            results = requests.get(
                url,
                headers=self.make_request_headers(self.env.user),
                timeout=default_timeout,
            ).json()
            error = results.get("error")
            if error:
                raise Exception(error.get("code", "Error"))
            drives = results.get("value")
            for drive in drives:
                if drive.get("name") == "SIOF":
                    return drive
            return None
        except Exception:
            raise
